@@ -38,8 +38,10 @@
 #' @param .data A data frame, data frame extension (e.g. a tibble), or a
 #'   lazy data frame (e.g. from dbplyr or dtplyr). See *Methods*, below, for
 #'   more details.
-#' @param ... <[`data-masking`][dplyr_data_masking]> Variables, or functions or
+#' @param ... <[`data-masking`][dplyr_data_masking]> Variables, or functions of
 #'   variables. Use [desc()] to sort a variable in descending order.
+#' @param .by_group If `TRUE`, will sort first by grouping variable. Applies to
+#'   grouped data frames only.
 #' @family single table verbs
 #' @examples
 #' arrange(mtcars, cyl, disp)
@@ -66,9 +68,6 @@ arrange <- function(.data, ..., .by_group = FALSE) {
   UseMethod("arrange")
 }
 
-#' @param .by_group If `TRUE`, will sort first by grouping variable. Applies to
-#'   grouped data frames only.
-#' @rdname arrange
 #' @export
 arrange.data.frame <- function(.data, ..., .by_group = FALSE) {
   dots <- enquos(...)
@@ -83,7 +82,9 @@ arrange.data.frame <- function(.data, ..., .by_group = FALSE) {
 
 # Helpers -----------------------------------------------------------------
 
-arrange_rows <- function(.data, dots) {
+arrange_rows <- function(.data, dots, error_call = caller_env()) {
+  error_call <- dplyr_error_call(error_call)
+
   if (length(dots) == 0L) {
     out <- seq_len(nrow(.data))
     return(out)
@@ -94,11 +95,13 @@ arrange_rows <- function(.data, dots) {
   })
 
   quosures <- map(dots, function(quosure) {
-    if (quo_is_call(quosure, "desc")) {
-      quosure <- new_quosure(
-        node_cadr(quo_get_expr(quosure)),
-        quo_get_env(quosure)
-      )
+    if (quo_is_call(quosure, "desc", ns = c("", "dplyr"))) {
+      expr <- quo_get_expr(quosure)
+      if (!has_length(expr, 2L)) {
+        abort("`desc()` must be called with exactly one argument.", call = error_call)
+      }
+
+      quosure <- new_quosure(node_cadr(expr), quo_get_env(quosure))
     }
     quosure
   })
@@ -120,14 +123,24 @@ arrange_rows <- function(.data, dots) {
     if (inherits(cnd, "dplyr:::mutate_error")) {
       # reverse the name mangling
       bullets <- gsub("^^--arrange_quosure_", "..", cnd$bullets, fixed = TRUE)
+      # only name bullets that aren't already named
+      names <- names2(bullets)
+      names[names == ""] <- "x"
+      bullets <- set_names(bullets, names)
+
+      # skip the parent as this has reworked the bullets
+      # and this would be confusing to have them
+      parent <- cnd$parent
     } else {
-      bullets <- c(x = conditionMessage(cnd))
+      parent <- cnd
+      bullets <- c()
     }
 
-    abort(c(
-      "arrange() failed at implicit mutate() step. ",
+    bullets <- c(
+      "Problem with the implicit `transmute()` step. ",
       bullets
-    ), class = "dplyr_error")
+    )
+    abort(bullets, call = error_call, parent = parent)
 
   })
 
