@@ -5,8 +5,9 @@
 #' which does the same thing for `NULL`s.
 #'
 #' @param ... <[`dynamic-dots`][rlang::dyn-dots]> Vectors. Inputs should be
-#'    recyclable (either be length 1 or same length as the longest vector) and
-#'    coercible to a common type.
+#'   recyclable (either be length 1 or same length as the longest vector) and
+#'   coercible to a common type. If data frames, they are coalesced column by
+#'   column.
 #' @return A vector the same length as the first `...` argument with
 #'   missing values replaced by the first non-missing value.
 #' @seealso [na_if()] to replace specified values with a `NA`.
@@ -34,15 +35,35 @@ coalesce <- function(...) {
   }
 
   values <- list2(...)
-  values <- vec_cast_common(!!!values)
-  values <- vec_recycle_common(!!!values)
+  values <- fix_call(vec_cast_common(!!!values))
+  values <- fix_call(vec_recycle_common(!!!values))
 
   x <- values[[1]]
   values <- values[-1]
 
+  if (is.array(x) && length(dim(x)) > 1) {
+    abort("Can't coalesce matrices.")
+  }
+  if (is.data.frame(x)) {
+    df_coalesce(x, values)
+  } else {
+    vec_coalesce(x, values)
+  }
+}
+
+vec_coalesce <- function(x, values) {
   for (i in seq_along(values)) {
     x_miss <- is.na(x)
     vec_slice(x, x_miss) <- vec_slice(values[[i]], x_miss)
   }
+
+  x
+}
+df_coalesce <- function(x, values) {
+  for (i in seq_along(x)) {
+    col_values <- map(values, `[[`, i)
+    x[[i]] <- vec_coalesce(x[[i]], col_values)
+  }
+
   x
 }
