@@ -1,7 +1,7 @@
 #' Select distinct rows by a selection of variables
 #'
 #' @description
-#' \Sexpr[results=rd, stage=render]{lifecycle::badge("superseded")}
+#' `r lifecycle::badge("superseded")`
 #'
 #' Scoped verbs (`_if`, `_at`, `_all`) have been superseded by the use of
 #' [across()] in an existing verb. See `vignette("colwise")` for details.
@@ -21,6 +21,7 @@
 #' The grouping variables that are part of the selection are taken
 #' into account to determine distinct rows.
 #'
+#' @keywords internal
 #' @examples
 #' df <- tibble(x = rep(2:5, each = 2) / 2, y = rep(2:3, each = 4) / 2)
 #'
@@ -42,7 +43,7 @@
 #' # ->
 #' distinct(df, across(everything(), round))
 distinct_all <- function(.tbl, .funs = list(), ..., .keep_all = FALSE) {
-  lifecycle::signal_superseded("1.0.0", "distinct_all()", "across()")
+  lifecycle::signal_stage("superseded", "distinct_all()")
   funs <- manip_all(.tbl, .funs, enquo(.funs), caller_env(), .include_group_vars = TRUE, ..., .caller = "distinct_all")
   if (!length(funs)) {
     funs <- syms(tbl_vars(.tbl))
@@ -52,7 +53,7 @@ distinct_all <- function(.tbl, .funs = list(), ..., .keep_all = FALSE) {
 #' @rdname distinct_all
 #' @export
 distinct_at <- function(.tbl, .vars, .funs = list(), ..., .keep_all = FALSE) {
-  lifecycle::signal_superseded("1.0.0", "distinct_at()", "across()")
+  lifecycle::signal_stage("superseded", "distinct_at()")
   funs <- manip_at(.tbl, .vars, .funs, enquo(.funs), caller_env(), .include_group_vars = TRUE, ..., .caller = "distinct_at")
   if (!length(funs)) {
     funs <- tbl_at_syms(.tbl, .vars, .include_group_vars = TRUE)
@@ -62,7 +63,7 @@ distinct_at <- function(.tbl, .vars, .funs = list(), ..., .keep_all = FALSE) {
 #' @rdname distinct_all
 #' @export
 distinct_if <- function(.tbl, .predicate, .funs = list(), ..., .keep_all = FALSE) {
-  lifecycle::signal_superseded("1.0.0", "distinct_if()", "across()")
+  lifecycle::signal_stage("superseded", "distinct_if()")
   funs <- manip_if(.tbl, .predicate, .funs, enquo(.funs), caller_env(), .include_group_vars = TRUE, ..., .caller = "distinct_if")
   if (!length(funs)) {
     funs <- tbl_if_syms(.tbl, .predicate, .include_group_vars = TRUE)
