@@ -90,6 +90,20 @@ test_that("mutate() handles symbol expressions", {
   expect_identical(df$x, res$y)
 })
 
+test_that("mutate() supports constants (#6056)", {
+  df <- data.frame(x = 1:10, g = rep(1:2, each = 5))
+  y <- 1:10
+  z <- 1:2
+
+  expect_error(df %>% mutate(y = !!y), NA)
+  expect_error(df %>% group_by(g) %>% mutate(y = !!y), NA)
+  expect_error(df %>% rowwise() %>% mutate(y = !!y), NA)
+
+  expect_error(df %>% mutate(z = !!z))
+  expect_error(df %>% group_by(g) %>% mutate(z = !!z))
+  expect_error(df %>% rowwise() %>% mutate(z = !!z))
+})
+
 # column types ------------------------------------------------------------
 
 test_that("glue() is supported", {
@@ -338,13 +352,13 @@ test_that(".keep = 'none' only keeps grouping variables", {
   expect_named(mutate(gf, z = 1, .keep = "none"), c("x", "z"))
 })
 
-test_that(".keep = 'none' prefers new order", {
+test_that(".keep = 'none' retains original ordering (#5967)", {
   df <- tibble(x = 1, y = 2)
-  expect_named(df %>% mutate(y = 1, x = 2, .keep = "none"), c("y", "x"))
+  expect_named(df %>% mutate(y = 1, x = 2, .keep = "none"), c("x", "y"))
 
   # even when grouped
   gf <- group_by(df, x)
-  expect_named(gf %>% mutate(y = 1, x = 2, .keep = "none"), c("y", "x"))
+  expect_named(gf %>% mutate(y = 1, x = 2, .keep = "none"), c("x", "y"))
 })
 
 test_that("can use .before and .after to control column position", {
@@ -358,6 +372,27 @@ test_that("can use .before and .after to control column position", {
   expect_named(mutate(df, x = 1, .after = y), c("x", "y"))
 })
 
+test_that(".keep and .before/.after interact correctly", {
+  df <- tibble(x = 1, y = 1, z = 1, a = 1, b = 2, c = 3) %>%
+    group_by(a, b)
+
+  expect_named(mutate(df, d = 1, x = 2, .keep = "none"), c("x", "a", "b", "d"))
+  expect_named(mutate(df, d = 1, x = 2, .keep = "none", .before = "a"), c("x", "d", "a", "b"))
+  expect_named(mutate(df, d = 1, x = 2, .keep = "none", .after = "a"), c("x", "a", "d", "b"))
+})
+
+test_that("dropping column with `NULL` then readding it retains original location", {
+  df <- tibble(x = 1, y = 2, z = 3, a = 4)
+  df <- group_by(df, z)
+
+  expect_named(mutate(df, y = NULL, y = 3, .keep = "all"), c("x", "y", "z", "a"))
+  expect_named(mutate(df, b = a, y = NULL, y = 3, .keep = "used"), c("y", "z", "a", "b"))
+  expect_named(mutate(df, b = a, y = NULL, y = 3, .keep = "unused"), c("x", "y", "z", "b"))
+
+  # It isn't treated as a "new" column
+  expect_named(mutate(df, y = NULL, y = 3, .keep = "all", .before = x), c("x", "y", "z", "a"))
+})
+
 test_that(".keep= always retains grouping variables (#5582)", {
   df <- tibble(x = 1, y = 2, z = 3) %>% group_by(z)
   expect_equal(
@@ -461,7 +496,7 @@ test_that("can suppress or catch warnings from the outside (#5675)", {
   # Check that caught warnings are instrumented. Requires
   # <https://github.com/wch/r-source/commit/688eaebf>.
   if (can_return_from_exit) {
-    expect_match(msg, "Problem with")
+    expect_match(msg, "Problem while")
   }
 })
 
@@ -476,92 +511,112 @@ test_that("mutate() propagates caller env", {
   expect_caller_env(mutate(mtcars, sig_caller_env()))
 })
 
+test_that("rowwise() + mutate(across()) correctly handles list columns (#5951)", {
+  tib <- tibble(a=list(1:2,3:4),c=list(NULL,NULL)) %>% rowwise()
+  expect_identical(
+    mutate(tib, sum = across(everything(),sum)),
+    mutate(tib, sum = across(where(is.list),sum))
+  )
+})
+
+test_that("mutate() fails on named empty arguments (#5925)", {
+  expect_error(
+    mutate(tibble(), bogus = )
+  )
+})
 
 # Error messages ----------------------------------------------------------
 
 test_that("mutate() give meaningful errors", {
-  tbl <- tibble(x = 1:2, y = 1:2)
+  expect_snapshot({
+    tbl <- tibble(x = 1:2, y = 1:2)
 
-  # setting column to NULL makes it unavailable
-  expect_snapshot(error = TRUE, tbl %>% mutate(y = NULL, a = sum(y)))
-  expect_snapshot(error = TRUE, tbl %>%
-    group_by(x) %>%
-    mutate(y = NULL, a = sum(y))
-  )
+    # setting column to NULL makes it unavailable
+    (expect_error(tbl %>% mutate(y = NULL, a = sum(y))))
+    (expect_error(tbl %>%
+                      group_by(x) %>%
+                      mutate(y = NULL, a = sum(y))
+    ))
 
-  # incompatible column type
-  expect_snapshot(error = TRUE, tibble(x = 1) %>% mutate(y = mean))
+    # incompatible column type
+    (expect_error(tibble(x = 1) %>% mutate(y = mean)))
 
-  # Unsupported type"
-  df <- tibble(g = c(1, 1, 2, 2, 2), x = 1:5)
-  expect_snapshot(error = TRUE, df %>% mutate(out = env(a = 1)))
-  expect_snapshot(error = TRUE, df %>%
-    group_by(g) %>%
-    mutate(out = env(a = 1))
-  )
-  expect_snapshot(error = TRUE, df %>%
-    rowwise() %>%
-    mutate(out = rnorm)
-  )
+    # Unsupported type"
+    df <- tibble(g = c(1, 1, 2, 2, 2), x = 1:5)
+    (expect_error(df %>% mutate(out = env(a = 1))))
+    (expect_error(df %>%
+                      group_by(g) %>%
+                      mutate(out = env(a = 1))
+    ))
+    (expect_error(df %>%
+                      rowwise() %>%
+                      mutate(out = rnorm)
+    ))
 
-  # incompatible types across groups
-  expect_snapshot(error = TRUE,
-    data.frame(x = rep(1:5, each = 3)) %>%
-      group_by(x) %>%
-      mutate(val = ifelse(x < 3, "foo", 2))
-  )
+    # incompatible types across groups
+    (expect_error(
+                    data.frame(x = rep(1:5, each = 3)) %>%
+                      group_by(x) %>%
+                      mutate(val = ifelse(x < 3, "foo", 2))
+    ))
 
-  expect_snapshot(error = TRUE,
-    tibble(a = 1:3, b=4:6) %>%
-      group_by(a) %>%
-      mutate(if(a==1) NULL else "foo")
-  )
+    (expect_error(
+                    tibble(a = 1:3, b=4:6) %>%
+                      group_by(a) %>%
+                      mutate(if(a==1) NULL else "foo")
+    ))
 
-  # incompatible size
-  expect_snapshot(error = TRUE,
-    data.frame(x = c(2, 2, 3, 3)) %>% mutate(int = 1:5)
-  )
-  expect_snapshot(error = TRUE,
-    data.frame(x = c(2, 2, 3, 3)) %>%
-      group_by(x) %>%
-      mutate(int = 1:5)
-  )
-  expect_snapshot(error = TRUE,
-    data.frame(x = c(2, 3, 3)) %>%
-      group_by(x) %>%
-      mutate(int = 1:5)
-  )
-  expect_snapshot(error = TRUE,
-    data.frame(x = c(2, 2, 3, 3)) %>%
+    # incompatible size
+    (expect_error(
+                    data.frame(x = c(2, 2, 3, 3)) %>% mutate(int = 1:5)
+    ))
+    (expect_error(
+                    data.frame(x = c(2, 2, 3, 3)) %>%
+                      group_by(x) %>%
+                      mutate(int = 1:5)
+    ))
+    (expect_error(
+                    data.frame(x = c(2, 3, 3)) %>%
+                      group_by(x) %>%
+                      mutate(int = 1:5)
+    ))
+    (expect_error(
+                    data.frame(x = c(2, 2, 3, 3)) %>%
+                      rowwise() %>%
+                      mutate(int = 1:5)
+    ))
+    (expect_error(
+                    tibble(y = list(1:3, "a")) %>%
+                      rowwise() %>%
+                      mutate(y2 = y)
+    ))
+    (expect_error(
+                    data.frame(x = 1:10) %>% mutate(y = 11:20, y = 1:2)
+    ))
+
+    # .data pronoun
+    (expect_error(
+                    tibble(a = 1) %>% mutate(c = .data$b)
+    ))
+    (expect_error(
+                    tibble(a = 1:3) %>%
+                      group_by(a) %>%
+                      mutate(c = .data$b)
+    ))
+
+    # obsolete data mask
+    lazy <- function(x) list(enquo(x))
+    res <- tbl %>%
       rowwise() %>%
-      mutate(int = 1:5)
-  )
-  expect_snapshot(error = TRUE,
-    tibble(y = list(1:3, "a")) %>%
-      rowwise() %>%
-      mutate(y2 = y)
-  )
-  expect_snapshot(error = TRUE,
-    data.frame(x = 1:10) %>% mutate(y = 11:20, y = 1:2)
-  )
+      mutate(z = lazy(x), .keep = "unused")
+    (expect_error(
+      eval_tidy(res$z[[1]])
+    ))
 
-  # .data pronoun
-  expect_snapshot(error = TRUE,
-    tibble(a = 1) %>% mutate(c = .data$b)
-  )
-  expect_snapshot(error = TRUE,
-    tibble(a = 1:3) %>%
-      group_by(a) %>%
-      mutate(c = .data$b)
-  )
 
-  # obsolete data mask
-  lazy <- function(x) list(enquo(x))
-  res <- tbl %>%
-    rowwise() %>%
-    mutate(z = lazy(x), .keep = "unused")
-  expect_snapshot(error = TRUE, eval_tidy(res$z[[1]]))
-
-  # Error that contains {
-  expect_snapshot(error = TRUE, tibble() %>% mutate(stop("{")))
+    # Error that contains {
+    (expect_error(
+      tibble() %>% mutate(stop("{"))
+    ))
+  })
 })
